package ch.ethz.icr.culturemodel;

import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.engine.Controller;
import uchicago.src.sim.engine.SimInit;
import uchicago.src.sim.gui.DisplayConstants;
import uchicago.src.sim.gui.DisplaySurface;
import uchicago.src.sim.gui.Object2DDisplay;
import uchicago.src.reflector.RangePropertyDescriptor;

import java.util.Iterator;

/**
 * Axelrod's Culture Model
 *
 * Based on Axelrod, The Complexity of Cooperation, Chap. 7.
 *
 * @author Luc Girardin
 * @author Lars-Erik Cederman
 * @version 2.0
 */
public class ModelGUI extends Model {
    // GUI variables
    public DisplaySurface dsurf;    // The display (grid) (provided by Repast)
    public OpenSequenceGraph graph; // The dynamic graph chart (provided by Repast)

    boolean countRegions;

    /**
     * The constructor of the GUI
     */
    public ModelGUI() {
        // Setting the order of appearance of parameters
        // to NON-alphabetic
        Controller.ALPHA_ORDER = false;
        Controller.CONSOLE_ERR = false;
        Controller.CONSOLE_OUT = false;
    }

    public void markRegion(Actor a, int numRegions) {
        // We mark the agent as one belonging to the given region
        a.region = numRegions;
        // Then we go through all its neighbors, too.
        Iterator it = a.neighbors.iterator();
        while (it.hasNext()) {
            Actor neighbor = (Actor) it.next();
            // If the neighbor is unmarked yet, and has the same culture as
            // our initial agent, we mark it (and its similar neighbors) as well.
            if ((neighbor.region == 0) && (a.similarTo(neighbor) == 1.0))
                markRegion(neighbor, numRegions);
        }
    }

    public int regionCounter() {
        // We clear the region field of all the agents first
        Iterator it = agentList.iterator();
        while (it.hasNext()) {
            Actor a = (Actor) it.next();
            a.region = 0;
        }

        // Then we go through all the agents again and
        // assign regional identity to all of them.
        int numRegions = 0;
        it = agentList.iterator();
        while (it.hasNext()) {
            Actor a = (Actor) it.next();
            // If the agent hasn't been reached yet, we start our next
            // region out of it
            if (a.region == 0) {
                numRegions++;
                markRegion(a, numRegions);
            }
        }

        return numRegions;
    }

    class Seq implements Sequence {
        // This is the method to be defined for Sequence
        public double getSValue() {
            // Returns the appropriate value
            return (double) regionCounter();
        }
    }

    /**
     * Initializing the GUI model
     */
    public void setup() {
        // Initializing the original model first
        super.setup();

        countRegions = true;

        // Specify the parameters to be displayed for setting by the user
        params = new String[]{"WorldSize", "NumFeatures", "NumTraits", "CountRegions"};

        RangePropertyDescriptor pdWorldSize = new RangePropertyDescriptor("WorldSize",
                5, 25, 5);
        descriptors.put("WorldSize", pdWorldSize);

        RangePropertyDescriptor pdNumFeatures = new RangePropertyDescriptor("NumFeatures",
                0, 25, 5);
        descriptors.put("NumFeatures", pdNumFeatures);

        RangePropertyDescriptor pdNumTraits = new RangePropertyDescriptor("NumTraits",
                0, 25, 5);
        descriptors.put("NumTraits", pdNumTraits);

        // Initializing the GUI-part
        // If there is already a graphics object from a previous run, we need
        // to delete it to clean up the screen.
        if (graph != null)
            graph.dispose();
        // If the display exists, let's dispose it!
        if (dsurf != null)
            dsurf.dispose();

        // Now it's time to create the display surface.
        // (For details see RePast's appropriate "How to" document.)
        dsurf = new DisplaySurface(this, "Culture Display");
        // Registering it as part of our GUI
        registerDisplaySurface("Main", dsurf);
    }

    /**
     * The method to build the Model's internals
     * Here we build the model instrumentation after the model itself has
     * been built.
     */
    public void buildModel() {
        // Build the original model first
        super.buildModel();

        // Setting the dimensions of each cell on the display (grid)
        DisplayConstants.CELL_WIDTH = 30;
        DisplayConstants.CELL_HEIGHT = 30;

        // Now create the 2D-display itself
        Object2DDisplay display = new Object2DDisplay(world);
        // Let's link our collection of the agents to it.
        // (The display will show players from this list.)
        display.setObjectList(agentList);
        // Adding the grid display to the display we have created before
        dsurf.addDisplayableProbeable(display, "Display");
        // Letting RePast know that we have this display up
        addSimEventListener(dsurf);
        // Displaying the display right away
        dsurf.display();

        // Create a sequence chart graph and set it up
        // (For details see Repast's appropriate "How to" document.)
        if (countRegions) {
            graph = new OpenSequenceGraph("Number of Regions", this);
            graph.setXRange(0, 100);
            graph.setYRange(0.0, (double) numActors);
            graph.setAxisTitles("Generations", "Number");
            // Add the sequences we have defined
            graph.addSequence("Regions", new Seq());
            // Display the graph right away
            graph.display();
            // Do the first update to it
            graph.step();
        }

    }

    /////////////////////////////////////////////////////////////////////////
    //  Iterated methods ////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////

    /**
     * The activity of the timestep
     * In each time step, first execute the model step and then update the
     * graph
     */
    public void step() {
        // Do the original model's step() method first
        super.step();

        // Do the GUI-part of it by updating the display
        if (getTickCount() % 100 == 0) {
            dsurf.updateDisplay();
            if (countRegions) {
                graph.step();
            }
        }
    }

    /////////////////////////////////////////////////////////////////////////
    // RePast Parameter Panel Methods ///////////////////////////////////////
    /////////////////////////////////////////////////////////////////////////

    // In the following we provide get/set methods for all the parameters
    // we listed in params (see the setup() method)

    public int getWorldSize() {
        return worldSize;
    }

    public void setWorldSize(int n) {
        worldSize = n;
    }

    public int getNumFeatures() {
        return numFeatures;
    }

    public void setNumFeatures(int n) {
        numFeatures = n;
    }

    public int getNumTraits() {
        return numTraits;
    }

    public void setNumTraits(int n) {
        numTraits = n;
    }

    public boolean getCountRegions() {
        return countRegions;
    }

    public void setCountRegions(boolean v) {
        countRegions = v;
    }


    /**
     * Creating and starting your model
     *
     * @param args
     */
    public static void main(String[] args) {
        SimInit init = new SimInit();
        // We MUST create a ModelGUI object instead of an instance of Model
        // as in the parent class, in order to get the GUI functionality.
        Model m = new ModelGUI();
        init.loadModel(m, null, false);
    }
}